@@ -191,7 +191,7 @@ cgiGetCheckbox(const char *name)	/* I - Name of form field */
   int		ret;			/* Return value */
 
 
-  ret = value && !_cups_strcasecmp(value, "checkbox");
+  ret = value && !_cups_strcasecmp(value, "checked");
 
   if (!ret && value)
   {
@@ -284,9 +284,11 @@ cgiGetTextfield(const char *name)	/* I - Name of form field */
 
     if (i < form_count)
       memmove(var, var + 1, (size_t)(form_count - i) * sizeof(_cgi_var_t));
+
+    value = NULL;
   }
 
-  return (value);
+  return (value ? strdup(value) : NULL);
 }
 
 
