@@ -80,7 +80,7 @@ cupsdAcceptClient(cupsd_listener_t *lis)/* I - Listener socket */
   * Make sure we don't have a full set of clients already...
   */
 
-  if (cupsArrayCount(Clients) == MaxClients)
+  if (MaxClients > 0 && cupsArrayCount(Clients) >= MaxClients)
     return;
 
   cupsdSetBusyState(1);
@@ -330,6 +330,13 @@ cupsdAcceptClient(cupsd_listener_t *lis)/* I - Listener socket */
     con->serverport = httpAddrPort(&(lis->address));
   }
 
+ /*
+  * Apply ServerHeader if any
+  */
+
+  if (ServerHeader)
+    httpSetDefaultField(con->http, HTTP_FIELD_SERVER, ServerHeader);
+
  /*
   * Add the connection to the array of active clients...
   */
@@ -456,7 +463,7 @@ cupsdCloseClient(cupsd_client_t *con)	/* I - Client to close */
       partial = 1;
 #endif /* HAVE_TLS */
 
-    if (partial)
+    if (partial && !httpError(con->http))
     {
      /*
       * Only do a partial close so that the encrypted client gets everything.
@@ -556,6 +563,7 @@ cupsdReadClient(cupsd_client_t *con)	/* I - Client to read from */
 {
   char			line[32768],	/* Line from client... */
 			locale[64],	/* Locale */
+			name[128],	/* Class/Printer name */
 			*ptr;		/* Pointer into strings */
   http_status_t		status;		/* Transfer status */
   ipp_state_t		ipp_state;	/* State of IPP transfer */
@@ -1138,16 +1146,29 @@ cupsdReadClient(cupsd_client_t *con)	/* I - Client to read from */
 	      }
 	      else if (!strncmp(con->uri, "/classes", 8))
 	      {
-	        if (strlen(con->uri) > 9 && !cupsdFindClass(con->uri + 9))
-	        {
-		  if (!cupsdSendError(con, HTTP_STATUS_NOT_FOUND, CUPSD_AUTH_NONE))
+		if (strlen(con->uri) > 9)
+		{
+		  if (con->uri[9] != '?')
 		  {
-		    cupsdCloseClient(con);
-		    return;
-		  }
+		    unsigned int i = 0;	// Array index
 
-		  break;
-	        }
+		    for (ptr = con->uri + 9; *ptr && *ptr != '?' && i < sizeof(name);)
+		      name[i++] = *ptr++;
+
+		    name[i] = '\0';
+
+		    if (!cupsdFindClass(name))
+		    {
+		      if (!cupsdSendError(con, HTTP_STATUS_NOT_FOUND, CUPSD_AUTH_NONE))
+		      {
+			cupsdCloseClient(con);
+			return;
+		      }
+
+		      break;
+		    }
+		  }
+		}
 
 		cupsdSetStringf(&con->command, "%s/cgi-bin/classes.cgi", ServerBin);
 		if (con->uri[8] && con->uri[9])
@@ -1165,16 +1186,29 @@ cupsdReadClient(cupsd_client_t *con)	/* I - Client to read from */
 	      }
               else if (!strncmp(con->uri, "/printers", 9))
 	      {
-	        if (strlen(con->uri) > 10 && !cupsdFindPrinter(con->uri + 10))
-	        {
-		  if (!cupsdSendError(con, HTTP_STATUS_NOT_FOUND, CUPSD_AUTH_NONE))
+		if (strlen(con->uri) > 10)
+		{
+		  if (con->uri[10] != '?')
 		  {
-		    cupsdCloseClient(con);
-		    return;
-		  }
+		    unsigned int i = 0;	// Array index
 
-		  break;
-	        }
+		    for (ptr = con->uri + 10; *ptr && *ptr != '?' && i < sizeof(name);)
+		      name[i++] = *ptr++;
+
+		    name[i] = '\0';
+
+		    if (!cupsdFindPrinter(name))
+		    {
+		      if (!cupsdSendError(con, HTTP_STATUS_NOT_FOUND, CUPSD_AUTH_NONE))
+		      {
+			cupsdCloseClient(con);
+			return;
+		      }
+
+		      break;
+		    }
+		  }
+		}
 
 		cupsdSetStringf(&con->command, "%s/cgi-bin/printers.cgi", ServerBin);
                 if (con->uri[9] && con->uri[10])
@@ -2119,9 +2153,6 @@ cupsdSendHeader(
     code = HTTP_STATUS_OK;
   }
 
-  if (ServerHeader)
-    httpSetField(con->http, HTTP_FIELD_SERVER, ServerHeader);
-
   if (code == HTTP_STATUS_METHOD_NOT_ALLOWED)
     httpSetField(con->http, HTTP_FIELD_ALLOW, "GET, HEAD, OPTIONS, POST, PUT");
 
